@@ -6,8 +6,8 @@ use FileHandle ();
 use File::Basename ();
 use Cwd ();
 
-#$Id: Registry.pm,v 1.46 1998/02/20 01:36:17 dougm Exp $
-$Apache::Registry::VERSION = (qw$Revision: 1.46 $)[1];
+#$Id: Registry.pm,v 1.47 1998/03/17 05:51:46 dougm Exp $
+$Apache::Registry::VERSION = (qw$Revision: 1.47 $)[1];
 
 $Apache::Registry::Debug ||= 0;
 # 1 => log recompile in errorlog
@@ -122,6 +122,7 @@ sub handler {
 	    compile($eval);
 	    if ($@) {
 		$r->log_error($@);
+		$@{$r->uri} = $@;
 		return SERVER_ERROR unless $Debug && $Debug & 2;
 		return Apache::Debug::dump($r, SERVER_ERROR);
 	    }
@@ -142,7 +143,7 @@ sub handler {
 	if($@) {
 	    $errsv = $@;
 	    $@ = ''; #XXX fix me, if we don't do this Apache::exit() breaks
-	    $@{"Apache::Registry"} = $errsv if $Apache::ERRSV_CAN_BE_HTTP;
+	    $@{$r->uri} = $errsv;
 	}
 
 	if($errsv) {
