@@ -4,7 +4,7 @@ BEGIN {
 
     use lib './t/docs';
     require "blib.pl" if -e "./t/docs/blib.pl";
-
+    $Apache::ServerStarting or warn "Server is not starting !?\n";
 }
 
 {
@@ -20,8 +20,15 @@ BEGIN {
     $s->warn("report any problems to server_admin $admin");
 }
 
-#$Apache::TestSIG = 1;
+#use HTTP::Status ();
+#use Apache::Symbol ();
+#Apache::Symbol->make_universal;
+
 $Apache::DoInternalRedirect = 1;
+$Apache::ERRSV_CAN_BE_HTTP  = 1;
+
+#warn "ServerStarting=$Apache::ServerStarting\n";
+#warn "ServerReStarting=$Apache::ServerReStarting\n";
 
 #use Apache::Debug level => 4;
 
@@ -31,6 +38,8 @@ if(defined &main::subversion) {
     die "mod_perl.pm is broken\n";
 }
 
+$ENV{KeyForPerlSetEnv} eq "OK" or warn "PerlSetEnv is broken\n";
+
 #test Apache::RegistryLoader
 {
     use Cwd ();
@@ -90,11 +99,19 @@ sub My::child_init {
     my $r = shift;
     my $s = $r->server;
     my $sa = $s->server_admin;
-    $s->warn("child_init for process $$, report any problems to $sa\n");
+    $s->warn("[notice] child_init for process $$, report any problems to $sa\n");
 }
 
 sub My::child_exit {
-    warn "child process $$ terminating\n";
+    warn "[notice] child process $$ terminating\n";
+}
+
+sub My::restart {
+    my $r = shift;
+    my $s = $r->server;
+    my $sa = $s->server_admin;
+    push @HTTP::Status::ISA, "Apache::Symbol";
+    HTTP::Status->undef_functions;
 }
 
 sub Apache::AuthenTest::handler {
@@ -117,15 +134,23 @@ sub Apache::AuthenTest::handler {
     return OK;                       
 }
 
+if(Apache->can_stack_handlers) {
+    Apache->push_handlers(PerlChildExitHandler => sub {
+	warn "[notice] push'd PerlChildExitHandler called, pid=$$\n";
+    });
+}
+
 END {
-    warn "END block called for startup.pl\n";
+    warn "[notice] END block called for startup.pl\n";
 }
 
 package Destruction;
 
 sub new { bless {} }
 
-sub DESTROY { warn "global object $global_object DESTROYed\n" }
+sub DESTROY { 
+    warn "[notice] Destruction->DESTROY called for \$global_object\n" 
+}
 
 #prior to 1.3b1 (and the child_exit hook), this object's DESTROY method would not be invoked
 $global_object = Destruction->new;
