@@ -1,10 +1,14 @@
 use Apache ();
+use Apache::Constants ();
 use strict;
 
-my $tests = 27;
+Apache->register_cleanup(sub {0});
+
+my $tests = 35;
 my $i;
 my $r = Apache->request;
 $r->content_type("text/plain");
+$r->content_languages([qw(en)]);
 $r->send_http_header;
 $r->print("1..$tests\n");
 
@@ -55,13 +59,17 @@ $r->header_out(ByeBye => undef);
 test ++$i, not $r->header_out("ByeBye");
 
 #content_encoding
-#content_language
+test ++$i, $r->content_languages;
 #no_cache
-#test ++$i, $r->uri;
-#test ++$i, $r->filename;
+test ++$i, $r->uri;
+test ++$i, $r->filename;
 #test ++$i, $r->path_info;
 #test ++$i, $r->query_string;
 
+#just make sure we can actually call these
+test ++$i, $r->satisfies || 1;
+test ++$i, $r->some_auth_required || 1;
+
 #dir_config
 
 my $c = $r->connection;
@@ -81,6 +89,9 @@ test ++$i, $s->server_admin;
 test ++$i, $s->server_hostname;
 test ++$i, $s->port;
 
+test ++$i, $r->module("Apache");
+test ++$i, not Apache->module("Not::A::Chance");
+test ++$i, Apache->module("Apache::Constants");
 
 
 
